@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -1164,37 +1164,47 @@ const OFT_info_t oft_info[OFT__N+1] =
     { OFT_UNKNOWN,
 	0,
 	IOM__IS_DEFAULT,
-	"?", 0, "\0", 0, "unkown file format" },
+	"?", 0, "\0", 0, "Unkown file format" },
 
     { OFT_PLAIN,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_LOADER,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_LOADER,
 	IOM_IS_IMAGE,
-	"ISO", "--iso", ".iso", 0, "plain file" },
+	"ISO", "--iso", ".iso", 0, "Plain file" },
 
-    { OFT_WDF,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_EXTEND|OFT_A_MODIFY,
+    { OFT_WDF1,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_WDF,
 	IOM_IS_IMAGE,
-	"WDF", "--wdf", ".wdf", 0, "Wii Disc Format" },
+	"WDF1", "--wdf1", ".wdf", ".wdf1", "Wii Disc Format v1" },
+
+    { OFT_WDF2,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_WDF,
+	IOM_IS_IMAGE,
+	"WDF2", "--wdf2", ".wdf", ".wdf2", "Wii Disc Format v2" },
 
     { OFT_CISO,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_MODIFY|OFT_A_NOSIZE|OFT_A_LOADER,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_MODIFY|OFT_A_NOSIZE|OFT_A_LOADER,
 	IOM_IS_IMAGE,
 	"CISO", "--ciso", ".ciso", ".wbi", "Compact ISO" },
 
     { OFT_WBFS,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_NOSIZE|OFT_A_LOADER,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_EXTEND|OFT_A_MODIFY|OFT_A_NOSIZE|OFT_A_LOADER,
 	IOM_IS_IMAGE,
 	"WBFS", "--wbfs", ".wbfs", 0, "Wii Backup File System" },
 
     { OFT_WIA,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_COMPR,
-	IOM_IS_WIA,
-	"WIA", "--wia", ".wia", 0, "compressed Wii ISO Archive" },
+	OFT_A_READ|OFT_A_CREATE|OFT_A_COMPR,
+	IOM_IS_COMPRESSED,
+	"WIA", "--wia", ".wia", 0, "Compressed Wii ISO Archive" },
+
+    { OFT_GCZ,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_COMPR,
+	IOM_IS_COMPRESSED,
+	"GCZ", "--gcz", ".gcz", 0, "Dolphins GameCube Zip" },
 
     { OFT_FST,
-	OFT_A_READ|OFT_A_WRITE|OFT_A_FST,
+	OFT_A_READ|OFT_A_CREATE|OFT_A_FST|OFT_A_DEST_EDIT,
 	IOM__IS_DEFAULT,
-	"FST", "--fst", "\0", 0, "extracted File System" },
+	"FST", "--fst", "\0", 0, "Extracted File System" },
 
     { OFT__N,
 	0,
@@ -1207,10 +1217,13 @@ const OFT_info_t oft_info[OFT__N+1] =
 const CommandTab_t ImageTypeTab[] =
 {
     { OFT_PLAIN,	"ISO",	"PLAIN",	0 },
-    { OFT_WDF,		"WDF",	0,		0 },
+    { OFT__WDF_DEF,	"WDF",	0,		0 },
+    { OFT_WDF1,		"WDF1",	"WDFV1",	0 },
+    { OFT_WDF2,		"WDF2",	"WDFV2",	0 },
     { OFT_CISO,		"CISO",	0,		0 },
     { OFT_WBFS,		"WBFS",	0,		0 },
-    { OFT_WIA,		"ISO",	0,		0 },
+    { OFT_WIA,		"WIA",	0,		0 },
+    { OFT_GCZ,		"GCZ",	"DOLPHIN",	0 },
     { OFT_FST,		"FST",	0,		1 },
     { 0,0,0,0 }
 };
@@ -1220,7 +1233,7 @@ const CommandTab_t ImageTypeTab[] =
 enumOFT CalcOFT ( enumOFT force, ccp fname_dest, ccp fname_src, enumOFT def )
 {
     if ( force > OFT_UNKNOWN && force < OFT__N )
-	return force;
+	return force == OFT__WDF_DEF ? ProposeOFT_WDF(def) : force;
 
     ccp fname = IsDirectory(fname_dest,true) ? fname_src : fname_dest;
     if (fname)
@@ -1229,11 +1242,14 @@ enumOFT CalcOFT ( enumOFT force, ccp fname_dest, ccp fname_src, enumOFT def )
 	if ( len >= 4 )
 	{
 	    if ( !strcasecmp(fname+len-4,".wdf") )
-		return OFT_WDF;
+		return ProposeOFT_WDF(def);
 
 	    if ( !strcasecmp(fname+len-4,".wia") )
 		return OFT_WIA;
 
+	    if ( !strcasecmp(fname+len-4,".gcz") )
+		return OFT_GCZ;
+
 	    if ( !strcasecmp(fname+len-4,".iso") )
 		return OFT_PLAIN;
 
@@ -1242,6 +1258,12 @@ enumOFT CalcOFT ( enumOFT force, ccp fname_dest, ccp fname_src, enumOFT def )
 
 	    if ( len >= 5 )
 	    {
+		if ( !strcasecmp(fname+len-5,".wdf1") )
+		    return OFT_WDF1;
+
+		if ( !strcasecmp(fname+len-5,".wdf2") )
+		    return OFT_WDF2;
+
 		if ( !strcasecmp(fname+len-5,".wbfs") )
 		    return OFT_WBFS;
 
@@ -1295,6 +1317,70 @@ static void ExtractSplitMap
     }
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// [[split]]
+
+static int split_seek ( File_t *f, off_t off )
+{
+    // returns:
+    //	 0: success
+    //	 1: failure
+    //   2: to big
+
+    enumError err = ExecSeekF(f,off);
+    PRINT("split_seek(%llx) err=%d, errno=%d\n",(u64)off,err,errno);
+    if (!err)
+	return 0;
+
+    const int syserr = errno;
+    return syserr == EINVAL || syserr == EFBIG ? 2 : 1;
+}
+
+//-----------------------------------------------------------------------------
+
+enumError XSetupAutoSplit ( XPARM File_t *f, enumOFT oft )
+{
+    // [[2do]] : seek() doesn't work on Cygwin!
+
+    ASSERT(f);
+    if ( !opt_auto_split || f->split_f || f->fd == -1 || !S_ISREG(f->st.st_mode) )
+	return ERR_OK;
+
+    const off_t off = 0x100000000ull;
+    int stat = split_seek(f,off);
+    split_seek(f,0);
+    if ( stat < 2 )
+	return ERR_OK;
+
+    off_t split_size;
+    uint split_factor;
+    if ( oft == OFT_PLAIN || oft == OFT_CISO || oft == OFT_WBFS )
+    {
+	split_size   = DEF_SPLIT_SIZE_ISO;
+	split_factor = DEF_SPLIT_FACTOR_ISO;
+    }
+    else
+    {
+	split_size   = DEF_SPLIT_SIZE;
+	split_factor = DEF_SPLIT_FACTOR;
+    }
+
+    stat = split_seek(f,off/2);
+    split_seek(f,0);
+    if ( stat > 0 )
+	split_size /= 2;
+
+    if ( split_factor > 0 )
+	split_size &= ~(split_factor-1);
+
+    PRINT("AUTO-SPLIT: %llx\n",(u64)split_size);
+    if ( verbose > 1 )
+	printf("   - Auto split at offset %llu (0x%llx)\n",
+		(u64)split_size, (u64)split_size );
+
+    return SetupSplitFile(f,oft,split_size);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 
 enumError XSetupSplitFile ( XPARM File_t *f, enumOFT oft, off_t split_size )
@@ -1394,7 +1480,7 @@ enumError XSetupSplitFile ( XPARM File_t *f, enumOFT oft, off_t split_size )
     }
     else
     {
-	f->split_filesize  = split_size ? split_size : DEF_SPLIT_SIZE;
+	f->split_filesize = split_size ? split_size : DEF_SPLIT_SIZE;
 	if ( DEF_SPLIT_FACTOR > 0 )
 	{
 	    f->split_filesize &= ~(DEF_SPLIT_FACTOR-1);
@@ -1703,12 +1789,12 @@ void DefineCachedAreaISO ( File_t * f, bool head_only )
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 
-enumError XAnalyzeWH ( XPARM File_t * f, WDF_Head_t * wh, bool print_err )
+enumError XAnalyzeWH ( XPARM File_t * f, wdf_header_t * wh0, bool print_err )
 {
-    ASSERT(wh);
+    DASSERT(wh0);
     TRACE("AnalyzeWH()\n");
 
-    if (memcmp(wh->magic,WDF_MAGIC,WDF_MAGIC_SIZE))
+    if (memcmp(wh0->magic,WDF_MAGIC,WDF_MAGIC_SIZE))
     {
 	TRACE(" - magic failed\n");
 	return ERR_NO_WDF;
@@ -1723,53 +1809,46 @@ enumError XAnalyzeWH ( XPARM File_t * f, WDF_Head_t * wh, bool print_err )
 		: ERR_WRONG_FILE_TYPE;
     }
 
-    const size_t wdf_head_size = AdjustHeaderWDF(wh);
+    wdf_header_t wh;
+    FixHeaderWDF(&wh,wh0,false);
 
- #if WDF2_ENABLED
-    if ( !wh->wdf_version || wh->wdf_compatible > WDF_VERSION )
- #else
-    if ( !wh->wdf_version || wh->wdf_version > WDF_VERSION )
- #endif
+    if ( !wh.wdf_version
+	|| wh.wdf_version > WDF_MAX_VERSION && wh.wdf_compatible > WDF_MAX_VERSION )
     {
-     #if WDF2_ENABLED
-	PRINT(" - wrong WDF version: %u,%u\n",wh->wdf_version,wh->wdf_compatible);
-     #else
-	PRINT(" - wrong WDF version: %u\n",wh->wdf_version);
-     #endif
+	PRINT(" - wrong WDF version: %u,%u\n",wh.wdf_version,wh.wdf_compatible);
 	return print_err
 		? PrintError( XERROR0, ERR_WDF_VERSION,
- #if WDF2_ENABLED
-			"Only WDF version 1..%u supported but not version %u.\n",
- #else
-			"Only WDF version %u supported but not version %u.\n",
- #endif
-			WDF_VERSION, wh->wdf_version )
+			"Only WDF versions 1..%u are supported but not version %u.\n",
+			WDF_MAX_VERSION, wh.wdf_version )
 		: ERR_WDF_VERSION;
     }
 
-    const u32 chunk_tab_size = wh->chunk_n * sizeof(WDF_Chunk_t);
-    if ( f->st.st_size < wh->chunk_off + WDF_MAGIC_SIZE + chunk_tab_size )
+    const u32 chunk_tab_size = wh.chunk_n * GetChunkSizeWDF(wh.wdf_version);
+    const u64 chunk_end = wh.chunk_off + WDF_MAGIC_SIZE + chunk_tab_size;
+    if ( chunk_end > f->st.st_size )
     {
 	// file size to short -> maybe a splitted file
 	XSetupSplitFile(XCALL f,OFT_UNKNOWN,0);
     }
 
-    if ( wh->chunk_off != wh->data_size + wdf_head_size
-	|| wh->chunk_off + WDF_MAGIC_SIZE + chunk_tab_size != f->st.st_size )
+ #if 0
+    if ( wh.chunk_off != wh.data_size + wh.head_size
+	|| wh.chunk_off + WDF_MAGIC_SIZE + chunk_tab_size != f->st.st_size )
     {
 	PRINT(" - file size error\n");
-	PRINT("   - %llx ? %llx = %llx + %zx\n",
-		(u64)wh->chunk_off, wh->data_size + wdf_head_size,
-		wh->data_size, wdf_head_size );
+	PRINT("   - %llx ? %llx = %llx + %x\n",
+		(u64)wh.chunk_off, wh.data_size + wh.head_size,
+		wh.data_size, wh.head_size );
 	PRINT("   - %llx + %x + %x = %llx ? %llx\n",
-		(u64)wh->chunk_off, WDF_MAGIC_SIZE, chunk_tab_size,
-		(u64)wh->chunk_off + WDF_MAGIC_SIZE + chunk_tab_size,
+		(u64)wh.chunk_off, WDF_MAGIC_SIZE, chunk_tab_size,
+		(u64)wh.chunk_off + WDF_MAGIC_SIZE + chunk_tab_size,
 		(u64)f->st.st_size );
 
 	return print_err
 		? PrintError( XERROR0, ERR_WDF_INVALID, "Invalid WDF file: %s\n",f->fname )
 		: ERR_WDF_INVALID;
     }
+ #endif
 
     TRACE(" - OK\n");
     return ERR_OK;
@@ -1969,6 +2048,31 @@ enumError XTellF ( XPARM File_t * f )
 
 ///////////////////////////////////////////////////////////////////////////////
 
+enumError ExecSeekF ( File_t * f, off_t off )
+{
+    bool failed;
+    if (f->fp)
+    {
+	const int stat = fseeko(f->fp,off,SEEK_SET);
+	failed = stat == -1;
+	noPRINT("ExecSeekF(%llx)/fseeko() -> stat=%d, errno=%d, failed=%d\n",
+		(u64)off, stat, errno, failed);
+    }
+    else if ( f->fd != -1 )
+    {
+	const off_t res = lseek(f->fd,off,SEEK_SET);
+	failed = res != off;
+	noPRINT("ExecSeekF(%llx)/lseek() -> off=%llx, errno=%d, failed=%d\n",
+		(u64)off, (u64)res, errno, failed );
+    }
+    else
+	failed = true;
+
+    return !failed ? ERR_OK : f->is_writing ? ERR_WRITE_FAILED : ERR_READ_FAILED;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 enumError XSeekF ( XPARM File_t * f, off_t off )
 {
     ASSERT(f);
@@ -2072,14 +2176,9 @@ enumError XSeekF ( XPARM File_t * f, off_t off )
     TRACE(TRACE_SEEK_FORMAT, "#F# SeekF()",
 		GetFD(f), GetFP(f), (u64)off, off < f->max_off ? " <" : "" );
 
-    const bool failed = f->fp
-			? fseeko(f->fp,off,SEEK_SET) == (off_t)-1
-			: f->fd == -1 || lseek(f->fd,off,SEEK_SET) == (off_t)-1;
-
-    enumError err;
-    if (failed)
+    enumError err = ExecSeekF(f,off);
+    if (err)
     {
-	err = f->is_writing ? ERR_WRITE_FAILED : ERR_READ_FAILED;
 	f->last_error = err;
 	if ( f->max_error < f->last_error )
 	    f->max_error = f->last_error;
@@ -2091,7 +2190,6 @@ enumError XSeekF ( XPARM File_t * f, off_t off )
     }
     else
     {
-	err = ERR_OK;
 	f->seek_count++;
 	if ( f->max_off < f->file_off )
 	    f->max_off = f->file_off;
@@ -3344,10 +3442,13 @@ int CalcSplitFilename ( char * buf, size_t buf_size, ccp path, enumOFT oft )
 
     if (!path)
 	path = "";
-    TRACE("CalcSplitFilename(%s,%d)\n",path,oft);
 
     size_t plen = strlen(path);
-    if ( plen > 0 && oft == OFT_WBFS )
+    const bool suppress_point = oft == OFT_WBFS
+		|| oft == OFT_PLAIN && plen >= 6 && !strcasecmp(path+plen-6,".part0");
+    PRINT("CalcSplitFilename(%s,%d) suppress_point=%d\n",path,oft,suppress_point);
+    
+    if ( plen > 0 && suppress_point )
 	plen--;
     if ( plen > max_path_len )
 	plen = max_path_len;
@@ -3369,7 +3470,7 @@ int CalcSplitFilename ( char * buf, size_t buf_size, ccp path, enumOFT oft )
 		*dest++	= *path++;
 	}
 
-	if ( oft != OFT_WBFS )
+	if (!suppress_point)
 	    *dest++ = '.';
 	*dest++ = '%';
 	*dest++ = '0';
