@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -40,6 +40,7 @@
 #include "lib-wdf.h"
 #include "lib-wia.h"
 #include "lib-ciso.h"
+#include "lib-gcz.h"
 #include "libwbfs.h"
 
 //
@@ -92,6 +93,7 @@ typedef struct IOData_t
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////			struct SuperFile_t		///////////////
 ///////////////////////////////////////////////////////////////////////////////
+// [[SuperFile_t]]
 
 typedef struct SuperFile_t
 {
@@ -118,7 +120,7 @@ typedef struct SuperFile_t
 	u32  progress_max_wd;		// max width used for progress output
 	ccp  progress_verb;		// default is "copied"
 	bool progress_summary;		// print summary (delayed) on closing
-	u64  progress_add_total;	// add this value to the total (managment data)
+	u64  progress_add_total;	// add this value to the total (management data)
 
 	u64  progress_last_done;	// last p_done value of PrintProgressSF() call
 	u64  progress_last_total;	// last p_total value of PrintProgressSF() call
@@ -136,6 +138,7 @@ typedef struct SuperFile_t
 	// read and write support
 
 	IOData_t iod;			// open file mode & read+write functions
+	enumOFT oft_orig;		// OFT of original source if source is FST
 	ReadFunc std_read_func;		// standard read function
 
 	// Wii disc support
@@ -146,10 +149,7 @@ typedef struct SuperFile_t
 
 	// WDF support
 
-	WDF_Head_t   wh;		// the WDF header
-	WDF_Chunk_t *wc;		// field with 'wc_size' elements
-	int wc_size;			// number of elements in 'wc'
-	int wc_used;			// number of used elements in 'wc'
+	wdf_controller_t * wdf;		// WDF controller
 
 	// WIA support
 
@@ -162,9 +162,13 @@ typedef struct SuperFile_t
 	// WBFS support (read only)
 
 	struct WBFS_t * wbfs;		// a WBFS
-	u32 wbfs_fragments;		// 0=unknown, >0:number if fragments
+	u32 wbfs_fragments;		// 0=unknown, >0:number of fragments
 	id6_t wbfs_id6;			// ID6 of wbfs inode
 
+	// GCZ support
+
+	GCZ_t * gcz;			// GCT header
+
 	// FST support
 	
 	struct WiiFst_t * fst;		// a FST
