@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -73,24 +73,6 @@
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#ifdef NEW_FEATURES // [[2do]]
- #define WDF2_ENABLED		  2	// 0:off, 1:read only, 2:write support
- #define WDF_VERSION		  1	// default version
- #define WDF_ALIGN		  1	// default alignment for WDF v2
- #define WDF_ALIGN_TEXT		"1"
- #define WDF_MAX_ALIGN		GiB	// max allowed WDF alignment
- #define WDF_MAX_ALIGN_TEXT	"1 GiB"
-#else
- #define WDF2_ENABLED		  0
- #define WDF_VERSION		  1	// default version
- #define WDF_ALIGN		  1	// default alignment for WDF v2
- #define WDF_ALIGN_TEXT		"1"
- #define WDF_MAX_ALIGN		GiB	// max allowed WDF alignment
- #define WDF_MAX_ALIGN_TEXT	"1 GiB"
-#endif
-
-///////////////////////////////////////////////////////////////////////////////
-
 typedef enum enumProgID
 {
 	PROG_UNKNOWN,
@@ -128,6 +110,11 @@ typedef enum enumRevID
 #define DEF_RECURSE_DEPTH	 10
 #define MAX_RECURSE_DEPTH	100
 
+///////////////////////////////////////////////////////////////////////////////
+
+#define M1(a) ( (typeof(a))~0 )
+#define IS_M1(a) ( (a) == (typeof(a))~0 )
+
 //
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////                       Setup                     ///////////////
@@ -166,6 +153,7 @@ enumError CheckEnvOptions ( ccp varname, check_opt_func );
  #define CheckCreated(f,d,e)	XCheckCreated	(__FUNCTION__,__FILE__,__LINE__,f,d,e)
  #define CreateFile(f,n,i,o)	XCreateFile	(__FUNCTION__,__FILE__,__LINE__,f,n,i,o)
  #define OpenStreamFile(f)	XOpenStreamFile	(__FUNCTION__,__FILE__,__LINE__,f)
+ #define SetupAutoSplit(f,m)	XSetupAutoSplit	(__FUNCTION__,__FILE__,__LINE__,f,m)
  #define SetupSplitFile(f,m,s)	XSetupSplitFile	(__FUNCTION__,__FILE__,__LINE__,f,m,s)
  #define CreateSplitFile(f,i)	XCreateSplitFile(__FUNCTION__,__FILE__,__LINE__,f,i)
  #define FindSplitFile(f,i,o)	XFindSplitFile	(__FUNCTION__,__FILE__,__LINE__,f,i,o)
@@ -201,6 +189,7 @@ enumError CheckEnvOptions ( ccp varname, check_opt_func );
  #define CheckCreated(f,d,e)	XCheckCreated	(f,d,e)
  #define CreateFile(f,n,i,o)	XCreateFile	(f,n,i,o)
  #define OpenStreamFile(f)	XOpenStreamFile	(f)
+ #define SetupAutoSplit(f,m)	XSetupAutoSplit	(f,m)
  #define SetupSplitFile(f,m,s)	XSetupSplitFile	(f,m,s)
  #define CreateSplitFile(f,i)	XCreateSplitFile(f,i)
  #define FindSplitFile(f,i,o)	XFindSplitFile	(f,i,o)
@@ -325,7 +314,7 @@ typedef enum enumIOMode
 {
 	IOM_IS_WBFS_PART	= 0x01, // is a WBFS partition
 	IOM_IS_IMAGE		= 0x02, // is a disc image (PLAIN, WDF, CISO, ...)
-	IOM_IS_WIA		= 0x04, // is a WIA file
+	IOM_IS_COMPRESSED	= 0x04, // is a WIA or GCZ file
 
 	IOM__IS_MASK		= 0x07,
 	IOM__IS_DEFAULT		= 0,
@@ -346,14 +335,17 @@ typedef enum enumOFT // open file mode
     OFT_UNKNOWN,		// not specified yet
 
     OFT_PLAIN,			// plain (ISO) file
-    OFT_WDF,			// WDF file
+    OFT_WDF1,			// WDFv1 file
+    OFT_WDF2,			// WDFv2 file
     OFT_CISO,			// CISO file
     OFT_WBFS,			// WBFS disc
     OFT_WIA,			// WIA file
+    OFT_GCZ,			// GCZ file
     OFT_FST,			// file system
 
     OFT__N,			// number of variants
-    OFT__DEFAULT = OFT_WBFS	// default value
+    OFT__DEFAULT = OFT_WBFS,	// default value
+    OFT__WDF_DEF = OFT_WDF2	// default WDF value
 
 } enumOFT;
 
@@ -361,14 +353,17 @@ typedef enum enumOFT // open file mode
 
 typedef enum attribOFT // OFT attributes
 {
-    OFT_A_READ		= 0x01,		// format can be read
-    OFT_A_WRITE		= 0x02,		// format can be written
-    OFT_A_MODIFY	= 0x04,		// format can be modified
-    OFT_A_EXTEND	= 0x08,		// format can be extended
-    OFT_A_FST		= 0x10,		// format is an extracted file system
-    OFT_A_COMPR		= 0x20,		// format uses compression
-    OFT_A_NOSIZE	= 0x40,		// format has no file size info
-    OFT_A_LOADER	= 0x80,		// used by USB/SD loaders
+    OFT_A_READ		= 0x0001,  // format can be read
+    OFT_A_CREATE	= 0x0002,  // format can be written
+    OFT_A_MODIFY	= 0x0004,  // format can be modified
+    OFT_A_EXTEND	= 0x0008,  // format can be extended
+    OFT_A_FST		= 0x0010,  // format is an extracted file system
+    OFT_A_COMPR		= 0x0020,  // format uses compression
+    OFT_A_NOSIZE	= 0x0040,  // format has no file size info
+    OFT_A_LOADER	= 0x0080,  // used by USB/SD loaders
+    OFT_A_DEST_EDIT	= 0x0100,  // if source, dest needs edit right
+
+    OFT_A_WDF		= 0x1000,  // is a WDF version
 
 } attribOFT;
 
@@ -393,8 +388,6 @@ typedef struct OFT_info_t
 extern const OFT_info_t oft_info[OFT__N+1];
 extern const CommandTab_t ImageTypeTab[];
 extern enumOFT output_file_type;
-extern uint opt_wdf_version;
-extern uint opt_wdf_align;
 extern int opt_truncate;
 
 enumOFT CalcOFT ( enumOFT force, ccp fname_dest, ccp fname_src, enumOFT def );
@@ -481,7 +474,7 @@ void InsertMemMapWrapper
 struct wd_disc_t;
 void InsertDiscMemMap
 (
-    MemMap_t		* mm,		// valid memore map pointer
+    MemMap_t		* mm,		// valid memory map pointer
     struct wd_disc_t	* disc		// valid disc pointer
 );
 
@@ -492,7 +485,7 @@ void InsertDiscMemMap
 uint FindMemMapHelper ( MemMap_t * mm, off_t off, off_t size );
 
 // Calculate overlaps. Return number of items with overlap.
-uint CalCoverlapMemMap ( MemMap_t * mm );
+uint CalcOverlapMemMap ( MemMap_t * mm );
 
 // Print out memory map
 void PrintMemMap ( MemMap_t * mm, FILE * f, int indent, ccp info_head );
@@ -551,7 +544,7 @@ uint CombineFileMaps
 
 typedef enum enumFileType
 {
-    // 1. file types
+	//--- basic file types
 
 	FT_UNKNOWN	=      0,  // not analyzed yet
 
@@ -561,7 +554,7 @@ typedef enum enumFileType
 	FT_ID_GC_ISO	= 0x0008,  // file is a GC ISO image
 	FT_ID_WII_ISO	= 0x0010,  // file is a WII ISO image
 
-	// special files
+	//--- special files
 
 	FT_ID_DOL	= 0x0020,  // file is a DOL file
 	FT_ID_CERT_BIN	= 0x0040,  // 'cert.bin' like file
@@ -576,28 +569,28 @@ typedef enum enumFileType
 	 FT__SPC_MASK	= 0x3fe0,  // mask of all special files
 	 FT__ID_MASK	= 0x3fff,  // mask of all 'FT_ID_' values
 
-    // 2. attributes
+	//--- attributes
 
 	FT_A_ISO	= 0x00010000,  // file is some kind of an ISO image
 	FT_A_GC_ISO	= 0x00020000,  // file is some kind of a GC ISO image
 	FT_A_WII_ISO	= 0x00040000,  // file is some kind of a WII ISO image
 
-	FT_A_WDISC	= 0x00100000,  // flag: specific disc of a WBFS file
-	FT_A_WDF	= 0x00200000,  // flag: file is a packed WDF
+	FT_A_WDISC	= 0x00080000,  // flag: specific disc of a WBFS file
+	FT_A_WDF1	= 0x00100000,  // flag: file is a packed WDF
+	FT_A_WDF2	= 0x00200000,  // flag: file is a packed WDF
 	FT_A_WIA	= 0x00400000,  // flag: file is a packed WIA
 	FT_A_CISO	= 0x00800000,  // flag: file is a packed CISO
-	FT_A_REGFILE	= 0x01000000,  // flag: file is a regular file
-	FT_A_BLOCKDEV	= 0x02000000,  // flag: file is a block device
-	FT_A_CHARDEV	= 0x04000000,  // flag: file is a block device
-	FT_A_SEEKABLE	= 0x08000000,  // flag: using of seek() is possible
-	FT_A_WRITING	= 0x10000000,  // is opened for writing
-	FT_A_PART_DIR	= 0x20000000,  // FST is a partition
+	FT_A_GCZ	= 0x01000000,  // flag: file is a packed GCZ
+	FT_A_REGFILE	= 0x02000000,  // flag: file is a regular file
+	FT_A_BLOCKDEV	= 0x04000000,  // flag: file is a block device
+	FT_A_CHARDEV	= 0x08000000,  // flag: file is a block device
+	FT_A_SEEKABLE	= 0x10000000,  // flag: using of seek() is possible
+	FT_A_WRITING	= 0x20000000,  // is opened for writing
+	FT_A_PART_DIR	= 0x40000000,  // FST is a partition
 
-	 FT__A_MASK	= 0x3ff70000,  // mask of all 'FT_A_' values
+	//--- special combinations
 
-    // 3. mask of all 'FT_ values
-
-	 FT__MASK	= FT__ID_MASK | FT__A_MASK,
+	FT_M_WDF	= FT_A_WDF1 | FT_A_WDF2
 
 } enumFileType;
 
@@ -662,6 +655,7 @@ FileAttrib_t * CopyFileAttribInode
 
 
 //-----------------------------------------------------------------------------
+// [[File_t]]
 
 typedef struct File_t
 {
@@ -779,7 +773,8 @@ enumError XOpenFileModify ( XPARM File_t * f, ccp fname, enumIOMode iomode );
 enumError XCreateFile     ( XPARM File_t * f, ccp fname, enumIOMode iomode, int overwrite );
 enumError XCheckCreated   ( XPARM             ccp fname, bool disable_errors, enumError err_code );
 enumError XOpenStreamFile ( XPARM File_t * f );
-enumError XSetupSplitFile ( XPARM File_t *f, enumOFT oft, off_t file_size );
+enumError XSetupAutoSplit ( XPARM File_t *f, enumOFT oft );
+enumError XSetupSplitFile ( XPARM File_t *f, enumOFT oft, off_t split_size );
 enumError XCreateSplitFile( XPARM File_t *f, uint split_idx );
 enumError XFindSplitFile  ( XPARM File_t *f, uint * index, off_t * off );
 
@@ -791,8 +786,8 @@ void ClearCache		 ( File_t * f );
 void DefineCachedArea    ( File_t * f, off_t off, size_t count );
 void DefineCachedAreaISO ( File_t * f, bool head_only );
 
-struct WDF_Head_t;
-enumError XAnalyzeWH ( XPARM File_t * f, struct WDF_Head_t * wh, bool print_err );
+struct wdf_header_t;
+enumError XAnalyzeWH ( XPARM File_t * f, struct wdf_header_t * wh, bool print_err );
 
 enumError StatFile ( struct stat * st, ccp fname, int fd );
 
@@ -810,6 +805,8 @@ enumError XWriteAtF	 ( XPARM File_t * f, off_t off, const void * iobuf, size_t c
 enumError XWriteZeroAtF	 ( XPARM File_t * f, off_t off,                     size_t count );
 enumError XZeroAtF	 ( XPARM File_t * f, off_t off,                     size_t count );
 
+enumError ExecSeekF ( File_t * f, off_t off );
+
 //-----------------------------------------------------------------------------
 // wrapper functions
 
@@ -1176,7 +1173,8 @@ enumError ScanSizeOptU32
 
 //-----------------------------------------------------------------------------
 
-extern int opt_split;
+extern int opt_auto_split;	// 0:off, 1:default, 2:enabled
+extern int opt_split;		// >0: spilt enabled
 extern u64 opt_split_size;
 
 // returns '1' on error, '0' else
@@ -1257,7 +1255,7 @@ typedef enum ShowMode
 	SHOW_UNUSED	= 0x00100000, // show unused areas
 	SHOW_OFFSET	= 0x00200000, // show offsets
 	SHOW_SIZE	= 0x00400000, // show size
-	
+
 	SHOW__ALL	= 0x007fffff,
 
 	//----- combinations
@@ -1287,8 +1285,10 @@ typedef enum ShowMode
 	SHOW_F_HEX	= 0x08000000, // prefer HEX,
 	SHOW_F__NUM	= 0x0f000000,
 
-	SHOW_F_HEAD	= 0x01000000, // print header lines
-	SHOW_F_PRIMARY	= 0x02000000, // print primary (unpatched) disc
+	SHOW_F_HEAD	= 0x10000000, // print header lines
+	SHOW_F_PRIMARY	= 0x20000000, // print primary (unpatched) disc
+
+	SHOW_F_SECTIONS	= 0x40000000, // print as sections if possible
 
 	//----- etc
 
