@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -1018,9 +1018,9 @@ enumError SetupReadWIA
     struct SuperFile_t	* sf		// file to setup
 )
 {
-    PRINT("#W# SetupReadWIA(%p) file=%d/%p, wc=%p wbfs=%p v=%s/%s\n",
+    PRINT("#W# SetupReadWIA(%p) file=%d/%p, wia=%p wbfs=%p v=%s/%s\n",
 		sf, GetFD(&sf->f), GetFP(&sf->f),
-		sf->wc, sf->wbfs,
+		sf->wia, sf->wbfs,
 		PrintVersionWIA(0,0,WIA_VERSION_READ_COMPATIBLE),
 		PrintVersionWIA(0,0,WIA_VERSION) );
     ASSERT(sf);
@@ -2190,7 +2190,7 @@ static enumError FinishSetupWriteWIA
 		    if ( size > max_size )
 			 size = max_size;
 		}
-		u32 n_sect;
+		u32 n_sect = 0;
 		wia_raw_data_t * rdata
 		    = need_raw_data( wia, 0x20, last_off, size, &n_sect );
 
