@@ -17,7 +17,7 @@
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
  *   Copyright (c) 2009 Kwiirk                                             *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -438,7 +438,7 @@ void wbfs_calc_geometry
 
     if (p->head)
     {
-	memset(p->head,0,sizeof(p->head));
+	memset(p->head,0,sizeof(*p->head));
 	p->head->magic		= wbfs_htonl(WBFS_MAGIC);
 	p->head->n_hd_sec	= wbfs_htonl(n_hd_sec);
 	p->head->hd_sec_sz_s	= hd_sec_sz_s;
