@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -224,9 +224,9 @@ typedef enum wd_sector_status_t
 {
     //----- cleared status
 
-    WD_SS_HASH_CLEARED		= 0x0001,   // hash area cleard with any constant
-    WD_SS_DATA_CLEARED		= 0x0002,   // data area cleard with any constant
-    WD_SS_SECTOR_CLEARED	= 0x0004,   // complete sector cleard with any constant
+    WD_SS_HASH_CLEARED		= 0x0001,   // hash area cleared with any constant
+    WD_SS_DATA_CLEARED		= 0x0002,   // data area cleared with any constant
+    WD_SS_SECTOR_CLEARED	= 0x0004,   // complete sector cleared with any constant
 					    // if set: WD_SS_HASH_CLEARED
 					    //	     & WD_SS_DATA_CLEARED are set too
 
@@ -289,7 +289,7 @@ typedef enum wd_scrubbed_t // scrubbing mode
 typedef enum wd_usage_t // usage table values
 {
     WD_USAGE_UNUSED,		// block is not used, always = 0
-    WD_USAGE_DISC,		// block is used for disc managment
+    WD_USAGE_DISC,		// block is used for disc management
     WD_USAGE_PART_0,		// index for first partition
 
     WD_USAGE__MASK	= 0x7f,	// mask for codes above
@@ -394,7 +394,8 @@ typedef enum wd_modify_t // objects to modify
 
     //--- summary
 
-    WD_MODIFY__ALL	= 0x01f,  // modify all
+    WD_MODIFY__TT	= WD_MODIFY_TICKET|WD_MODIFY_TMD,
+    WD_MODIFY__ALL	= 0x01f, // modify all
     WD_MODIFY__MASK	= WD_MODIFY__ALL | WD_MODIFY__AUTO | WD_MODIFY__ALWAYS,
 
 } wd_modify_t;
@@ -566,6 +567,7 @@ typedef struct wd_part_t
     struct wd_disc_t	* disc;		// pointer to disc
     wd_memmap_t		patch;		// patching data
 
+
     //----- partition status
 
     bool		is_loaded;	// true if this partition info was loaded
@@ -579,6 +581,7 @@ typedef struct wd_part_t
     wd_sector_status_t	sector_stat;	// sector status: OR'ed for different sectors
     bool		scrub_test_done;// true if scrubbing test already done
 
+
     //----- to do status
 
     bool		sign_ticket;	// true if ticket must be signed
@@ -946,7 +949,7 @@ enumError wd_read_raw
     u32			disc_offset4,	// disc offset/4
     void		* dest_buf,	// destination buffer
     u32			read_size,	// number of bytes to read 
-    wd_usage_t		usage_id	// not 0: mark usage usage_tab with this value
+    wd_usage_t		usage_id	// not 0: mark usage usage_table with this value
 );
 
 //-----------------------------------------------------------------------------
@@ -976,7 +979,7 @@ enumError wd_read_part
 (
     wd_part_t		* part,		// valid pointer to a disc partition
     u32			data_offset4,	// partition data offset/4
-    void		* dest_buf,	// estination buffer
+    void		* dest_buf,	// destination buffer
     u32			read_size,	// number of bytes to read 
     bool		mark_block	// true: mark block in 'usage_table'
 );
@@ -1142,6 +1145,18 @@ bool wd_part_has_h3
     wd_part_t		*part		// valid disc partition pointer
 );
 
+//-----------------------------------------------------------------------------
+
+uint wd_disc_is_encrypted
+(
+    // return
+    //    0: no partition is encrypted
+    //    N: some (=N) partitions are encrypted, but not all
+    // 1000: all partitions are encrypted
+    
+    wd_disc_t		* disc		// disc to check, valid
+);
+
 //
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////		    interface: get partition		///////////////
@@ -1890,6 +1905,29 @@ bool wd_patch_part_system // result = true if something changed
     u64			system		// new system id (IOS)
 );
 
+///////////////////////////////////////////////////////////////////////////////
+
+typedef struct wd_patch_main_t
+{
+    wd_disc_t	*disc;		// valid pointer to disc
+    wd_part_t	*part;		// valid pointer to partition, if file found
+
+    bool	patch_main;	// true: patch 'sys/main.dol'
+    bool	patch_staticr;	// true: patch 'files/rel/staticr.rel'
+
+    wd_memmap_item_t *main;	// NULL or found 'sys/main.dol'
+    wd_memmap_item_t *staticr;	// NULL or found 'files/rel/staticr.rel'
+}
+wd_patch_main_t;
+
+int wd_patch_main
+(
+    wd_patch_main_t	*pm,		// result only, will be initialized
+    wd_disc_t		*disc,		// valid disc
+    bool		patch_main,	// true: patch 'sys/main.dol'
+    bool		patch_staticr	// true: patch 'files/rel/staticr.rel'
+);
+
 //
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////		     interface: relocation		///////////////
