@@ -16,7 +16,7 @@
  *   This file is part of the WIT project.                                 *
  *   Visit http://wit.wiimm.de/ for project details and sources.           *
  *                                                                         *
- *   Copyright (c) 2009-2013 by Dirk Clemens <wiimm@wiimm.de>              *
+ *   Copyright (c) 2009-2015 by Dirk Clemens <wiimm@wiimm.de>              *
  *                                                                         *
  ***************************************************************************
  *                                                                         *
@@ -59,7 +59,7 @@
 
 typedef enum FileMode { FMODE_WDF, FMODE_WIA, FMODE_CISO, FMODE_WBI } FileMode;
 ccp file_mode_name[] = { ".wdf", ".wia", ".ciso", ".wbi", 0 };
-enumOFT file_mode_oft[] = { OFT_WDF, OFT_WIA, OFT_CISO, OFT_CISO, 0 };
+enumOFT file_mode_oft[] = { OFT__WDF_DEF, OFT_WIA, OFT_CISO, OFT_CISO, 0 };
 
 enumCommands the_cmd	= CMD_PACK;
 FileMode file_mode	= FMODE_WDF;
@@ -123,9 +123,14 @@ static void help_exit ( bool xmode )
 
 static void version_exit()
 {
-    printf("%s%s\n%.*s",
-	progname, TITLE,
-	(int)strlen(default_settings)-1, default_settings );
+    if ( brief_count > 1 )
+	fputs( VERSION "\n", stdout );
+    else if (brief_count)
+	fputs( VERSION " r" REVISION " " SYSTEM "\n", stdout );
+    else 
+	printf("%s%s\n%.*s",
+		progname, TITLE,
+		(int)strlen(default_settings)-1, default_settings );
     exit(ERR_OK);
 }
 
@@ -213,7 +218,7 @@ enumError CloseInOut ( SuperFile_t * fi, SuperFile_t * fo,
 	if ( prev_err < err )
 	     prev_err = err;
     }
-    
+
     TRACE("CloseInOut() returns %d\n",prev_err);
     return prev_err;
 }
@@ -255,7 +260,7 @@ enumError CatRaw ( SuperFile_t * fi, SuperFile_t * fo,
     TRACE("CatRaw()\n");
     DASSERT(fi);
     DASSERT( fo || out_fname );
-    
+
     enumError err = ERR_OK;
     SuperFile_t fo_local;
 
@@ -423,6 +428,8 @@ enumError CatWDF ( ccp fname, SuperFile_t * fo, ccp out_fname,
 	return ERR_OK;
     }
 
+    //--- wdf
+
     SuperFile_t fo_local;
     if ( !err && !fo )
     {
@@ -435,9 +442,15 @@ enumError CatWDF ( ccp fname, SuperFile_t * fo, ccp out_fname,
 
     if (!err)
     {
+	// [[wdf]] [[wdf2]] optimization like c++ example
+
+	wdf_controller_t *wdf = fi.wdf;
+	if (!wdf)
+	    ERROR0(ERR_INTERNAL,0);
+
 	int i;
 	u64 last_off = 0;
-	for ( i=0; i < fi.wc_used; i++ )
+	for ( i=0; i < wdf->chunk_used; i++ )
 	{
 	    if ( SIGINT_level > 1 )
 	    {
@@ -445,7 +458,7 @@ enumError CatWDF ( ccp fname, SuperFile_t * fo, ccp out_fname,
 		break;
 	    }
 
-	    WDF_Chunk_t *wc = fi.wc + i;
+	    wdf2_chunk_t *wc = wdf->chunk + i;
 	    u64 zero_count = wc->file_pos - last_off;
 	    if ( zero_count > 0 )
 	    {
@@ -485,7 +498,7 @@ enumError cmd_cat ( bool ignore_raw )
     SuperFile_t out;
     InitializeSF(&out);
     enumError err = OpenOutput(&out,0,0,"CONCATENATE files:");
-    
+
     ParamList_t * param;
     for ( param = first_param; !err && param; param = param->next )
 	err = CatWDF(param->arg,&out,0,ignore_raw,false);
@@ -556,7 +569,7 @@ enumError cmd_pack()
 ///////////////			   cmd cmp			///////////////
 ///////////////////////////////////////////////////////////////////////////////
 
-enumError cmd_cmp()
+enumError cmd_cmp() // cmd_diff()
 {
     if ( n_param < 1 )
 	return ERROR0(ERR_SYNTAX,"Missing source file!");
@@ -591,7 +604,7 @@ enumError cmd_cmp()
     SetupDiff(&diff,long_count);
     OpenDiffSource(&diff,&f1,&f2,true);
     err = DiffRawSF(&diff);
-    
+
     enumError err2 = CloseDiff(&diff);
     ResetSF(&f1,0);
     ResetSF(&f2,0);
@@ -658,7 +671,7 @@ enumError ciso_dump ( FILE *f, CISO_Head_t * ch, File_t *df, ccp fname )
 				? iso_file_usage : min_file_size;
 
     //----- print header
-    
+
     u8 * m = (u8*)ch->magic;
     fprintf(f,"  %-18s:         \"%s\" = %02x-%02x-%02x-%02x\n",
 		"Magic", wd_print_id(m,4,0), m[0], m[1], m[2], m[3] );
@@ -879,6 +892,113 @@ enumError wia_dump ( FILE *f, File_t *df, ccp fname )
 
 ///////////////////////////////////////////////////////////////////////////////
 
+enumError gcz_dump ( FILE *f, GCZ_Head_t * gh1, File_t *df, ccp fname )
+{
+    ASSERT(gh1);
+    ASSERT(df);
+    ASSERT(fname);
+
+    if (testmode)
+    {
+	fprintf(f," - WOULD dump GCZ %s\n",fname);
+	ResetFile(df,false);
+	return ERR_OK;
+    }
+
+    GCZ_Head_t gh;
+    const bool valid = IsValidGCZ(gh1,sizeof(*gh1),df->st.st_size,&gh);
+
+    fprintf(f,"\nGCZ dump of file %s\n\n",fname);
+    if (!valid)
+	fprintf(f,"  ** Invalid image! **\n\n");
+    else if ( gh.block_size != GCZ_DEF_BLOCK_SIZE )
+	fprintf(f,"  ** Not standard block size (%s)! **\n\n",
+		wd_print_size_1024(0,0,gh.block_size,false) );
+
+    fprintf(f,
+	"  Magic:             %08x\n"
+	"  GCZ type:        %10u\n"
+	"  Compressed size: %10llu = %9llx/hex\n"
+	"  Image size:      %10llu = %9llx/hex\n"
+	"  Block size:      %10u = %9x/hex\n"
+	"  Number of blocks:%10u = %9x/hex\n"
+	,gh.magic
+	,gh.sub_type
+	,gh.compr_size ,gh.compr_size
+	,gh.image_size ,gh.image_size
+	,gh.block_size ,gh.block_size
+	,gh.num_blocks ,gh.num_blocks
+	);
+
+    const uint list_size = 12 * gh.num_blocks;
+    u64 *offset = MALLOC(list_size);
+    //u32 *checksum = (u32*)(offset+gh.num_blocks);
+    enumError stat = ReadAtF(df,sizeof(gh),offset,list_size);
+    if (stat)
+	return stat;
+
+    u64 compr_size = 0;
+    uint blk, n_compr = 0, n_zero = 0;
+    for ( blk = 0; blk < gh.num_blocks; blk++ )
+    {
+	s64 blk_off  = le64(offset+blk);
+	u32 blk_size = ( blk == gh.num_blocks - 1
+				? gh.compr_size
+				: le64(offset+blk+1) ) - blk_off;
+
+	if ( blk_size == 39 && gh.block_size == GCZ_DEF_BLOCK_SIZE )
+	{
+	    n_zero++;
+	    //printf("zero: %5x %u\n",blk,blk_size);
+	}
+	else if ( blk_off >= 0 )
+	{
+	    n_compr++;
+	    compr_size += blk_size;
+	    //printf("compr: %5x\n",blk);
+	}
+    }
+
+    uint n_uncompr = gh.num_blocks - n_compr - n_zero;
+    uint ave = n_compr ? compr_size /  n_compr : 0;
+    fprintf(f,"     uncompressed: %10u = %9x/hex\n",n_uncompr,n_uncompr);
+    if (n_zero)
+	fprintf(f,"     zero filled:  %10u = %9x/hex\n",n_zero,n_zero);
+    fprintf(f,
+	"     compressed:   %10u = %9x/hex\n"
+	"  Average compr.size:%8u = %9x/hex\n"
+	"\n"
+	,n_compr ,n_compr
+	,ave, ave
+	);
+
+    ResetFile(df,false);
+    return ERR_OK;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+
+static u64 PrintAlignWC ( FILE *f, ccp title, u64 num )
+{
+    u64 m;
+    if (num)
+    {
+	m = 1;
+	while (!(num&m))
+	    m <<= 1;
+    }
+    else
+	m = 0;
+
+    fprintf(f,"    %-17s : %s\n",
+		title,
+		m ? wd_print_size_1024(0,0,m,true) : "   0" );
+    return m;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 enumError wdf_dump ( FILE *f, ccp fname )
 {
     ASSERT(fname);
@@ -889,7 +1009,7 @@ enumError wdf_dump ( FILE *f, ccp fname )
     if (err)
 	return err;
 
-    WDF_Head_t wh;
+    wdf_header_t wh;
     const int CISO_MAGIC_SIZE = 4;
     ASSERT( CISO_MAGIC_SIZE < sizeof(wh) );
     err = ReadAtF(&df,0,&wh,CISO_MAGIC_SIZE);
@@ -912,6 +1032,20 @@ enumError wdf_dump ( FILE *f, ccp fname )
 	return ciso_dump(f,&ch,&df,fname);
     }
 
+    if ( le32(&wh) == GCZ_MAGIC_NUM )
+    {
+	GCZ_Head_t gh;
+	const uint magic_size = 4;
+	memcpy(&gh,&wh,magic_size);
+	err = ReadAtF(&df,magic_size,(char*)&gh+magic_size,sizeof(gh)-magic_size);
+	if (err)
+	{
+	    ResetFile(&df,false);
+	    return err;
+	}
+	return gcz_dump(f,&gh,&df,fname);
+    }
+
     if (!memcmp(&wh,WIA_MAGIC,WIA_MAGIC_SIZE))
 	return wia_dump(f,&df,fname);
 
@@ -922,26 +1056,25 @@ enumError wdf_dump ( FILE *f, ccp fname )
 	return ERR_OK;
     }
 
+    //---------------
+
     fprintf(f,"\nWDF dump of file %s\n",fname);
 
-    err = ReadAtF(&df,CISO_MAGIC_SIZE,(char*)&wh+CISO_MAGIC_SIZE,sizeof(wh)-CISO_MAGIC_SIZE);
+    err = ReadAtF(&df,CISO_MAGIC_SIZE,(char*)&wh+CISO_MAGIC_SIZE,
+			sizeof(wh)-CISO_MAGIC_SIZE);
     if (err)
     {
 	ResetFile(&df,false);
 	return err;
     }
-    
+
+    wdf_header_t wh_fixed;
     ConvertToHostWH(&wh,&wh);
-    if ( wh.wdf_version == 1 )
-    {
-	wh.wdf_compatible	= 1;
-	wh.wdf_head_size	= WDF_VERSION1_SIZE;
-	wh.align_factor		= 0;
-	wh.chunk_size_factor	= 0;
-    }
+    const bool fixed = FixHeaderWDF(&wh_fixed,&wh,true);
     AnalyzeWH(&df,&wh,false); // needed for splitting support!
 
-    fprintf(f,"\n  Header:\n\n");
+    fprintf(f,"\n  File header:\n");
+
     u8 * m = (u8*)wh.magic;
     fprintf(f,"    %-18s: \"%s\"  %02x %02x %02x %02x  %02x %02x %02x %02x\n",
 		"Magic",
@@ -954,56 +1087,86 @@ enumError wdf_dump ( FILE *f, ccp fname )
 	return ERROR0(ERR_WDF_INVALID,"Wrong magic: %s\n",fname);
     }
 
-    const int head_size = AdjustHeaderWDF(&wh);
-
     #undef PRINT32
     #undef PRINT64
-    #undef RANGE
-    #define PRINT32(a) fprintf(f,"    %-18s: %10x/hex =%11d\n",#a,wh.a,wh.a)
-    #define PRINT64(a) fprintf(f,"    %-18s: %10llx/hex =%11lld\n",#a,wh.a,wh.a)
+    #undef PRINT32F
+    #define PRINT32(a,t) fprintf(f,"    %-18s: %10x/hex =%11d\n",t,wh.a,wh.a)
+    #define PRINT64(a,t) fprintf(f,"    %-18s: %10llx/hex =%11lld\n",t,wh.a,wh.a)
+    #define PRINT32F(a,t) if (wh_fixed.a!=wh.a) \
+		fprintf(f,"    %-18s: %10x/hex =%11d\n",t,wh_fixed.a,wh_fixed.a)
 
-    PRINT32(wdf_version);
-    PRINT32(wdf_compatible);
-    PRINT32(wdf_head_size);
-    PRINT32(align_factor);
-    PRINT64(file_size);
+    PRINT32(wdf_version,	"WDF version");
+    PRINT32(wdf_compatible,	"WDF compatible");
+    PRINT32(head_size,		"Head size");
+    PRINT32(align_factor,	"Alignment factor");
+    PRINT64(file_size,		"Image size");
     fprintf(f,"    %-18s: %10llx/hex =%11lld  %4.2f%%\n",
-		" - WDF file size ",
-		(u64)df.st.st_size, (u64)df.st.st_size,  100.0 * df.st.st_size / wh.file_size );
-    PRINT64(data_size);
-    PRINT32(chunk_size_factor);
-    PRINT32(chunk_n);
-    PRINT64(chunk_off);
+		"  WDF file size ",
+		(u64)df.st.st_size, (u64)df.st.st_size,
+		100.0 * df.st.st_size / wh.file_size );
+    PRINT64(data_size,	"Data size");
+    PRINT32(reserved,	"-reserved-");
+    PRINT32(chunk_n,	"Number of chunks");
+    PRINT64(chunk_off,	"Chunk data offset");
+
+    const uint chunk_entry_size = GetChunkSizeWDF(wh.wdf_version);
+    const uint chunk_table_size = chunk_entry_size * wh.chunk_n;
+
+    fprintf(f,"    %-18s: %10x/hex =%11d\n",
+		" Chunk entry size", chunk_entry_size, chunk_entry_size );
+    fprintf(f,"    %-18s: %10x/hex =%11d\n",
+		" Chunk table size", chunk_table_size, chunk_table_size );
 
     //--------------------------------------------------
 
-    fprintf(f,"\n  File Parts:\n\n");
+    if (fixed)
+    {
+	fprintf(f,"\n  Auto fixed header:\n");
+
+	PRINT32F(wdf_version,	"WDF version");
+	PRINT32F(wdf_compatible,"WDF compatible");
+	PRINT32F(head_size,	"Head size");
+	PRINT32F(align_factor,	"Alignment factor");
+	PRINT32F(reserved,	"-reserved-");
+
+	memcpy(&wh,&wh_fixed,sizeof(wh));
+    }
+
+    //--------------------------------------------------
+
+ #if 0
+    fprintf(f,"\n  Memory map of file:%21s%18s\n","offset range","[ data size]");
+
 
     int ec = 0; // error count
     prev_val = 0;
 
-    const int chunk_size = wh.chunk_n*sizeof(WDF_Chunk_t);
-    
-    ec += print_range(f, "Header",	head_size,			head_size );
+    ec += print_range(f, "Header",	wh.head_size,			wh.head_size );
     ec += print_range(f, "Data",	wh.chunk_off,			wh.data_size );
-    ec += print_range(f, "Chunk-Magic",	wh.chunk_off+WDF_MAGIC_SIZE,	WDF_MAGIC_SIZE );
-    ec += print_range(f, "Chunk-Table",	df.st.st_size,			chunk_size );
-    fprintf(f,"\n");
+    ec += print_range(f, "Chunk magic",	wh.chunk_off+WDF_MAGIC_SIZE,	WDF_MAGIC_SIZE );
+    ec += print_range(f, "Chunk table",	df.st.st_size,			chunk_table_size );
+    fputc('\n',f);
 
     if (ec)
     {
 	ResetFile(&df,false);
 	return ERROR0(ERR_WDF_INVALID,"Invalid data: %s\n",fname);
     }
+ #endif
 
     if ( wh.chunk_n > 1000000 )
     {
 	ResetFile(&df,false);
-	return ERROR0(ERR_INTERNAL,"Too much chunk enties: %s\n",fname);
+	return ERROR0(ERR_INTERNAL,"Too many chunk entries: %s\n",fname);
     }
 
     //--------------------------------------------------
 
+    MemMap_t mm;
+    InitializeMemMap(&mm);
+    MemMapItem_t *mi = InsertMemMap(&mm,0,wh.head_size);
+    snprintf(mi->info,sizeof(mi->info),"WDF header v%u",wh.wdf_version);
+
     char magic[WDF_MAGIC_SIZE];
     err = ReadAtF(&df,wh.chunk_off,magic,sizeof(magic));
     if (err)
@@ -1011,6 +1174,8 @@ enumError wdf_dump ( FILE *f, ccp fname )
 	ResetFile(&df,false);
 	return ERROR0(ERR_READ_FAILED,"ReadF error: %s\n",fname);
     }
+    mi = InsertMemMap(&mm,wh.chunk_off,sizeof(magic));
+    StringCopyS(mi->info,sizeof(mi->info),"magic of chunk list");
 
     if (memcmp(magic,WDF_MAGIC,WDF_MAGIC_SIZE))
     {
@@ -1018,26 +1183,94 @@ enumError wdf_dump ( FILE *f, ccp fname )
 	return ERROR0(ERR_WDF_INVALID,"Wrong chunk table magic: %s\n",fname);
     }
 
-    WDF_Chunk_t *w, *wc = MALLOC(chunk_size);
-    err = ReadF(&df,wc,chunk_size);
+    u8 *wc_data =  MALLOC(chunk_table_size);
+    err = ReadF(&df,wc_data,chunk_table_size);
     if (err)
     {
 	ResetFile(&df,false);
-	FREE(wc);
-	return ERROR0(ERR_READ_FAILED,"ReadF error: %s\n",fname);
+	FREE(wc_data);
+	return ERROR0(ERR_READ_FAILED,"Can't read chunk table: %s\n",fname);
     }
+    mi = InsertMemMap(&mm,wh.chunk_off+sizeof(magic),chunk_table_size);
+    snprintf(mi->info,sizeof(mi->info),"%u chunk elements",wh.chunk_n);
+    u64 eof = mi->off + mi->size;
+
+    wdf2_chunk_t *wc = (wdf2_chunk_t*)wc_data;
+    ConvertToHostWC(wc,wc_data,wh.wdf_version,wh.chunk_n);
+
+    u64 align_data_off = 0, align_data_size = 0, align_file_pos = 0;
+    u64 max_data = wh.head_size, min_data = wh.head_size;
+    int idx;
+    wdf2_chunk_t *w;
+    for ( idx = 0, w = wc; idx < wh.chunk_n; idx++, w++ )
+    {
+	align_data_off  |= w->data_off;
+	align_data_size |= w->data_size;
+	align_file_pos  |= w->file_pos;
+	
+	if ( !idx || min_data > w->data_off )
+	     min_data = w->data_off;
+	const u64 data_end = w->data_off + w->data_size;
+	if ( max_data < data_end )
+	     max_data = data_end;
+    }
+    mi = InsertMemMap(&mm,min_data,max_data-min_data);
+    snprintf(mi->info,sizeof(mi->info),"%u chunk data blocks",wh.chunk_n);
+
+    //--------------------------------------------------
+    // split file?
+
+    if ( eof < max_data )
+	eof = max_data;
+
+    if ( eof > df.st.st_size )
+	SetupSplitFile(&df,OFT_WDF2,eof);
+
+    char ch;
+    err = ReadAtF(&df,max_data-1,&ch,1);
+    if (err)
+    {
+	ResetFile(&df,false);
+	FREE(wc_data);
+	return ERROR0(ERR_READ_FAILED,"Can't read last data byte: %s\n",fname);
+    }
+
+    mi = InsertMemMap(&mm,df.st.st_size,0);
+    if ( df.split_used > 1 )
+	snprintf(mi->info,sizeof(mi->info),
+	    "-- end of %u joined files --", df.split_used );
+    else
+	StringCopyS(mi->info,sizeof(mi->info),"-- end of file --");
+
+
+    //--------------------------------------------------
+
+    fprintf(f,"\n  Data alignments:\n");
+    PrintAlignWC(f,"WDF header",	wh.align_factor);
+    PrintAlignWC(f,"Calculated",	align_data_off | align_data_size | align_file_pos );
+    PrintAlignWC(f," Data offset+size",	align_data_off | align_data_size );
+    PrintAlignWC(f,"  Data offset",	align_data_off );
+    PrintAlignWC(f,"  Data size",	align_data_size  );
+    PrintAlignWC(f," File offset+size",	align_file_pos | align_data_size );
+    PrintAlignWC(f,"  File offset",	align_file_pos );
+
+    uint n_overlap = CalcOverlapMemMap(&mm);
+    fprintf(f,"\n  Memory map of file (hex values)%s:\n\n",
+		n_overlap ? " => INVALID!" : "" );
+    PrintMemMap(&mm,f,4,0);
+    fputc('\n',f);
+
+    //--------------------------------------------------
 
     if (opt_chunk)
     {
 	fprintf(f,
-	    "  Chunk Table:\n\n"
+	    "  Chunk Table (hex values):\n\n"
 	    "    idx        WDF file offset  data len     virtual ISO offset  hole size\n"
 	    "   ------------------------------------------------------------------------\n");
 
 	int idx;
-	for ( idx = 0, w = wc; idx < wh.chunk_n; idx++, w++ )
-	    ConvertToHostWC(w,w);
-
+	wdf2_chunk_t *w;
 	for ( idx = 0, w = wc; idx < wh.chunk_n; idx++, w++ )
 	{
 	    fprintf(f,"%6d. %10llx..%10llx %9llx %10llx..%10llx  %9llx\n",
@@ -1045,12 +1278,13 @@ enumError wdf_dump ( FILE *f, ccp fname )
 		    w->data_off, w->data_off + w->data_size-opt_minus1,
 		    w->data_size,
 		    w->file_pos, w->file_pos + w->data_size - opt_minus1,
-		    idx+1 < wh.chunk_n ? w[1].file_pos - w->file_pos - w->data_size : 0llu );
+		    idx+1 < wh.chunk_n
+			    ? w[1].file_pos - w->file_pos - w->data_size : 0llu );
 	}
-
-	putchar('\n');
+	fputc('\n',f);
     }
 
+    FREE(wc_data);
     ResetFile(&df,false);
     return ERR_OK;
 }
@@ -1062,7 +1296,7 @@ enumError cmd_dump()
     SuperFile_t sf;
     InitializeSF(&sf);
     enumError err = OpenOutput(&sf,0,0,"DUMP WDF and CISO data strutures:");
-    
+
     ParamList_t * param;
     for ( param = first_param; !err && param; param = param->next )
 	err = wdf_dump(sf.f.fp,param->arg);
@@ -1104,7 +1338,12 @@ enumError CheckOptions ( int argc, char ** argv )
 	case GO_LONG:		opt_chunk = true; long_count++; break;
 	case GO_MINUS1:		opt_minus1 = 1; break;
 
-	case GO_WDF:		file_mode = FMODE_WDF; break;
+	//case GO_WDF:		file_mode = FMODE_WDF; break; // [[wdf2]]
+	case GO_WDF:		file_mode = FMODE_WDF; SetModeWDF(0,optarg); break;
+	case GO_WDF1:		file_mode = FMODE_WDF; SetModeWDF(1,optarg); break;
+	case GO_WDF2:		file_mode = FMODE_WDF; SetModeWDF(2,optarg); break;
+	case GO_ALIGN_WDF:	err += ScanOptAlignWDF(optarg,0); break;
+
 	case GO_WIA:		file_mode = FMODE_WIA;
 					err += ScanOptCompression(false,optarg);
 					break;
@@ -1112,12 +1351,6 @@ enumError CheckOptions ( int argc, char ** argv )
 	case GO_WBI:		file_mode = FMODE_WBI; break;
 	case GO_SUFFIX:		opt_suffix = optarg; break;
 
-    #if WDF2_ENABLED > 1
-	case GO_WDF1:		file_mode = FMODE_WDF; SetWDF2Mode(1,0); break;
-	case GO_WDF2:		file_mode = FMODE_WDF; SetWDF2Mode(2,optarg); break;
-	case GO_WDF_ALIGN:	err += ScanOptWDFAlign(optarg); break;
-    #endif
-
 	case GO_DEST:		SetDest(optarg,false); break;
 	case GO_DEST2:		SetDest(optarg,true); break;
 	case GO_STDOUT:		opt_stdout = true; break;
@@ -1125,6 +1358,8 @@ enumError CheckOptions ( int argc, char ** argv )
 	case GO_OVERWRITE:	opt_overwrite = true; break;
 	case GO_PRESERVE:	opt_preserve = true; break;
 
+	case GO_AUTO_SPLIT:	opt_auto_split = 2; opt_split = 0; break;
+	case GO_NO_SPLIT:	opt_auto_split = opt_split = 0; break;
 	case GO_SPLIT:		opt_split++; break;
 	case GO_SPLIT_SIZE:	err += ScanOptSplitSize(optarg); break;
 	case GO_PREALLOC:	err += ScanPreallocMode(optarg); break;
@@ -1190,7 +1425,7 @@ enumError CheckCommand ( int argc, char ** argv )
     argv += optind;
 
     TRACE("CheckCommand(%d,) optind=%d\n",argc+optind,optind);
-    
+
     if ( argc > 0 && **argv == '+' )
     {
 	int cmd_stat;
